@@ -33,6 +33,10 @@
 #include <sys/extattr.h>
 #endif
 
+#ifdef SOLARIS
+#include <sys/attr.h>
+#endif
+
 /* FIXME: are the ACL includes really neccessary here ? */
 #ifdef HAVE_SOLARIS_ACLS
 #include <sys/acl.h>
@@ -77,6 +81,11 @@ enum {
 #define XATTR_REPLACE 0x2       /* set value, fail if attr does not exist */
 #endif
 
+#ifdef SOLARIS
+#define SMB_ATTR_PREFIX "SUNWsmb:"
+#define SMB_ATTR_PREFIX_LEN (sizeof (SMB_ATTR_PREFIX) - 1)
+#endif
+
 /* Names for our Extended Attributes adouble data */
 #define AD_EA_META "org.netatalk.Metadata"
 #define AD_EA_RESO "org.netatalk.ResourceFork"
@@ -90,10 +99,12 @@ ssize_t sys_lgetxattr (const char *path, const char *name, void *value, size_t s
 ssize_t sys_fgetxattr (int filedes, const char *name, void *value, size_t size);
 ssize_t sys_listxattr (const char *path, char *list, size_t size);
 ssize_t sys_llistxattr (const char *path, char *list, size_t size);
-ssize_t sys_flistxattr (int filedes, char *list, size_t size);
+/* ssize_t sys_flistxattr (int filedes, char *list, size_t size); */
+ssize_t sys_flistxattr (int filedes, const char *path, char *list, size_t size);
 int sys_removexattr (const char *path, const char *name);
 int sys_lremovexattr (const char *path, const char *name);
-int sys_fremovexattr (int filedes, const char *name);
+/* int sys_fremovexattr (int filedes, const char *name); */
+int sys_fremovexattr (int filedes, const char *path, const char *name);
 int sys_setxattr (const char *path, const char *name, const void *value, size_t size, int flags);
 int sys_lsetxattr (const char *path, const char *name, const void *value, size_t size, int flags);
 int sys_fsetxattr (int filedes, const char *name, const void *value, size_t size, int flags);
