@@ -9,6 +9,7 @@
 #include <sys/param.h>
 #include <grp.h>
 #include <sys/types.h>
+#include <stdbool.h>
 
 #ifdef HAVE_NETDB_H
 #include <netdb.h>  /* this isn't header-protected under ultrix */
@@ -116,6 +117,10 @@ struct afp_options {
     mode_t umask;
     mode_t save_mask;
     gid_t admingid;
+    bool force_user;
+    uid_t force_uid;
+    bool force_group;
+    gid_t force_gid;
     int    volnamelen;
     /* default value for winbind authentication */
     char *ntdomain, *ntseparator, *addomain;
@@ -146,7 +151,8 @@ typedef struct AFPObj {
     char oldtmp[AFPOBJ_TMPSIZ + 1], newtmp[AFPOBJ_TMPSIZ + 1];
     void *uam_cookie; /* cookie for uams */
     struct session_info  sinfo;
-    uid_t uid; 	/* client running user id */
+    uid_t uid;  /* client login user id */
+    uid_t euid; /* client effective process user id */
     int ipc_fd; /* anonymous PF_UNIX socket for IPC with afpd parent */
     gid_t *groups;
     int ngroups;
